# Path from where to find files
INPUT_PATH = './input'

# Folder inside input that holds non-template files
CONTENT_DIR_NAME = 'content'

# Folder in which to output files
OUTPUT_PATH = './output'

# List of css files to bundle, order counts, use None for all css files in css folder
CSS_FILES = None

# Removes current output folder before generating
CLEAN_OUTPUT = True

# For log messages
import logging
LOG_FORMAT = '[%(levelname)s] %(message)s'
LOG_LEVEL = logging.DEBUG

# To enable escaping by default on Jinja2 prints
AUTOESCAPE=False



from pathlib import Path

from sssimp import APP_DIR

def mkdir(path):
    """
    Creates a directory or the parent directory if `path` is not a directory.
    Safely ignores if already exists
    """
    path = Path(path)
    if not path.is_dir():
        path = path.parent
    path.mkdir(exist_ok=True, parents=True)


def path_strip(path, parent = APP_DIR):
    """Strips `parent` from `path`"""
    path = str(path)
    path = path.removeprefix(str(parent))
    path = path.removeprefix('/')
    return path



from collections import namedtuple
import functools
from io import StringIO
import json
from markdown import Markdown, Extension
from markdown.inlinepatterns import SimpleTagInlineProcessor

import sssimp
from sssimp.generators.html import Page, PAGES
from sssimp import jinja
from sssimp.utils import path_strip


class MarkdownPage(Page):
    def __init__(self, src, target):
        super().__init__(src, target)
        with open(self.src) as f:
            self.content = f.read()
        self.vars['plain_text'] = markdown_to_text(self.content)
        result = markdown_to_html(self.content)
        self.meta = {
            key: json.loads(value[-1][1:])
            if value[-1].startswith('=')
            else value[-1]
            for key, value in result.meta.items()
        }
        self.meta.setdefault('template', f'{self.src.parent.name}.html')
        self.vars['meta'] = self.meta
        self.vars['markdown'] = result.html

    def get_template(self):
        return jinja.get_template(self.meta['template'])

    @property
    def title(self):
        if 'title' in self.meta:
            return self.meta['title']
        result = self.target.stem
        for char in '-_':
            result = result.replace(char, ' ')
        return result.capitalize()


class StrikeSubExtension(Extension):
    @staticmethod
    def _add_pattern(md, char, tag):
        proc = SimpleTagInlineProcessor(r'(\{0}\{0})(.+?)(\{0}\{0})'.format(char), tag)
        md.inlinePatterns.register(proc, tag, 200)

    def extendMarkdown(self, md):
        self._add_pattern(md, '~', 'del')
        self._add_pattern(md, '_', 'ins')


def unmark_element(element, stream=None):
    if stream is None:
        stream = StringIO()
    if element.text:
        stream.write(element.text)
    for sub in element:
        unmark_element(sub, stream)
    if element.tail:
        stream.write(element.tail)
    return stream.getvalue()
Markdown.output_formats['plain'] = unmark_element


def markdown_to_html(text):
    HtmlWithMeta = namedtuple('HtmlWithMeta', ('html', 'meta'))
    markdown = Markdown(
        extensions=[
            # Builtin extensions
            # cf https://python-markdown.github.io/extensions/ (in order)
            'abbr', 'fenced_code', 'footnotes', 'admonition', 'codehilite', 'meta',
            'smarty', 'toc',
            # Custom extensions
            StrikeSubExtension(),
        ],
        extension_configs={'smarty': {'smart_angled_quotes': True}},
        output_format='html',
    )
    return HtmlWithMeta(markdown.convert(text), markdown.Meta)

def markdown_to_text(text):
    markdown = Markdown(
        extensions=[
            'abbr', 'meta', 'admonition',
            StrikeSubExtension()
        ],
        output_format='plain',
    )
    markdown.stripTopLevelTags = False
    result = markdown.convert(text)
    result = result.replace('[TOC]', '')
    return result


def prepare():
    for file in sssimp.CONTENT_DIR.glob('**/*.md'):
        as_html = path_strip(file.with_suffix('.html'), sssimp.CONTENT_DIR)
        target = sssimp.OUTPUT_DIR / as_html
        PAGES.add(MarkdownPage(src=file, target=target))
        sssimp.IGNORE_ASSETS.add(str(file))



import functools
from pathlib import Path
from jinja2 import Environment, FileSystemLoader

from . import config

__version__ = '0.0.9'

APP_DIR = Path(__file__).parent
INPUT_DIR = Path(config.INPUT_PATH)
CONTENT_DIR = INPUT_DIR / config.CONTENT_DIR_NAME
OUTPUT_DIR = Path(config.OUTPUT_PATH)
IGNORE_ASSETS = set()

jinja = Environment(
    loader=FileSystemLoader('input/templates', followlinks=True),
    autoescape=config.AUTOESCAPE,
    trim_blocks=True,
    lstrip_blocks=True,
    extensions=['jinja2.ext.do'],
)

def filter(f):
    jinja.filters[f.__name__] = f
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        return f(*args, **kwargs)
    return wrapper

from . import filters



import logging
import functools
import sssimp
from sssimp import jinja
from sssimp.utils import mkdir, path_strip


PAGES = set()


class Page:
    def __init__(self, src, target):
        self.src = src
        self.target = target
        self.vars = {
            'page': self,
        }
        self.meta = None

    def __str__(self):
        return str(self.target)

    @property
    def created_at(self):
        return self.stat.st_ctime

    @property
    def last_modified(self):
        return self.stat.st_mtime

    @property
    @functools.cache
    def stat(self):
        return self.src.stat()

    @property
    def href(self):
        return path_strip(self.target, sssimp.OUTPUT_DIR)

    @property
    def name(self):
        return self.target.name

    @property
    def parent(self):
        return self.target.parent

    def get_template(self):
        with open(self.src) as f:
            content = f.read()
        return jinja.from_string(content)

    @functools.cache
    def render(self):
        return self.get_template().render(**self.vars)

    def write(self, target=None):
        target = target or self.target
        mkdir(self.target)
        with open(self.target, 'w') as f:
            logging.info(f'Generating {self.target}')
            f.write(self.render())


def prepare():
    for file in sssimp.CONTENT_DIR.glob('**/*.html'):
        target = sssimp.OUTPUT_DIR / path_strip(file, sssimp.CONTENT_DIR)
        PAGES.add(Page(src=file, target=target))
        sssimp.IGNORE_ASSETS.add(str(file))


def main():
    jinja.globals['PAGES'] = PAGES
    for page in PAGES:
        page.write()



import logging
import sssimp
from sssimp import config, jinja
from sssimp.utils import mkdir, path_strip


BUNDLE_FILE = 'bundle.css'
OUT_FILE = sssimp.OUTPUT_DIR / BUNDLE_FILE
CSS_DIR = sssimp.INPUT_DIR / 'css'
css_files = None

def prepare():
    global css_files
    jinja.globals['BUNDLE_FILE'] = BUNDLE_FILE
    css_files = list(
        [CSS_DIR / file for file in config.CSS_FILES]
        if config.CSS_FILES else
        CSS_DIR.glob('**/*.css')
    )
    jinja.globals['BUNDLE_TIME'] = max(x.stat().st_ctime for x in css_files)

def main():
    mkdir(OUT_FILE)
    with open(OUT_FILE, 'w') as out:
        for file in css_files:
            relative_path = path_strip(file, CSS_DIR)
            logging.info(f'Bundling {relative_path}')
            print(f'/** bundle:{relative_path} **/', file=out)
            with open(file) as css_file:
                out.write(css_file.read())
            print(file=out)
